package c.l.blockchain.directory;

import io.reactivex.Flowable;
import leigh.mecha.log.MechaLogger;
import leigh.mecha.log.MechaLoggerFactory;
import leigh.mecha.util.StringAccumulatorV2;
import leigh.mecha.util.UniversalJob;
import org.web3j.protocol.Web3j;
import org.web3j.protocol.core.DefaultBlockParameterName;
import org.web3j.protocol.core.methods.response.EthBlock;
import org.web3j.protocol.http.HttpService;

/**
 * This daemon implements the LeighChain directory service.
 *
 * @author C. Alexander Leigh
 */
public class DirectoryDaemon {
    private static final MechaLogger logger = MechaLoggerFactory.getLogger(DirectoryDaemon.class);

    public static void main(String[] args) {
        UniversalJob.banner(logger, "DirectoryDaemon");

        if (args.length != 1) {
            System.err.println("Usage: DirectoryDaemon url");
            System.err.println("Example: DirectoryDaemon http://localhost:8545");
            System.exit(UniversalJob.RET_BADARGS);
        }

        Web3j web3j = Web3j.build(new HttpService(args[0]));

        Flowable<EthBlock> subscription =
                web3j.replayPastAndFutureBlocksFlowable(DefaultBlockParameterName.EARLIEST, true);

        System.out.println("BLOCK_ID,TIMESTAMP,MINER,SIZE");

        subscription.blockingForEach(ethBlock -> {
            EthBlock.Block b = ethBlock.getBlock();

            StringAccumulatorV2 sa = new StringAccumulatorV2(",");
            sa.push(String.valueOf(b.getNumber()));
            sa.push(String.valueOf(b.getTimestamp()));
            sa.push(b.getMiner());
            sa.push(b.getSize().toString());

            System.out.println(sa.asString());
        });
    }
}
