package c.l.blockchain.directory;

import leigh.mecha.log.MechaLogger;
import leigh.mecha.log.MechaLoggerFactory;

import java.time.Instant;
import java.util.HashMap;

/**
 * This class implements the {@link DirectoryStore} interface using in-memory storage.
 *
 * @author C. Alexander Leigh
 */
public class RamDirectoryStore implements DirectoryStore {
    private static final MechaLogger logger = MechaLoggerFactory.getLogger(RamDirectoryStore.class);

    private final HashMap<String,Identity> nameIdx = new HashMap<>();
    private final HashMap<String,Identity> addrIdx = new HashMap<>();

    @Override
    public Identity getIdentityByName(String name, Instant when) {
        Identity id = nameIdx.get(name);

        if(id==null) return null;

        if(!id.isNameClaimValid(when)) {
            nameIdx.remove(name);
            return null;
        }
        return id;
    }

    @Override
    public Identity getIdentityByAddr(String address) {
        return addrIdx.get(address);
    }

    public void putIdentity(Identity id) {
        logger.info("Storing new identity: {}", id);
        addrIdx.put(id.getAddress(), id);
        String nm = id.getName();
        if(nm!=null) {
            nameIdx.put(nm, id);
        }
    }

    @Override
    public String toString() {
        return "RamDirectoryStore{" +
                "nameIdx=" + nameIdx +
                ", addrIdx=" + addrIdx +
                '}';
    }
}
