package leigh.infra.dnsd;

import leigh.mecha.log.MechaLogger;
import leigh.mecha.log.MechaLoggerFactory;
import org.xbill.DNS.Message;
import org.xbill.DNS.SimpleResolver;

import java.io.IOException;
import java.net.DatagramPacket;
import java.net.DatagramSocket;
import java.net.UnknownHostException;

public class DNSService implements Runnable {
    private static final MechaLogger logger = MechaLoggerFactory.getLogger(DNSService.class);
    private static final int UDP_SIZE = 512;
    private final int port;
    private final SimpleResolver resolver;
    private final NameStore nameStore = new EmergencyNameStore();

    DNSService(int port) throws UnknownHostException {
        this.port = port;
        resolver = new SimpleResolver("8.8.8.8");
    }

    public void run() {
        try {
            DatagramSocket socket = new DatagramSocket(port);
            //noinspection InfiniteLoopStatement
            while (true)
                process(socket);
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    private void process(DatagramSocket socket) throws IOException {
        byte[] in = new byte[UDP_SIZE];
        DatagramPacket indp = new DatagramPacket(in, UDP_SIZE);
        socket.receive(indp);
        Message request = new Message(in);

        Message reply = nameStore.doQuery(request);

        if (reply == null)
            reply = resolver.send(request);

        byte[] resp = reply.toWire();
        DatagramPacket outdp = new DatagramPacket(resp, resp.length,
                indp.getAddress(), indp.getPort());
        socket.send(outdp);
    }
}