package leigh.infra.dnsd;

import org.xbill.DNS.Record;
import org.xbill.DNS.*;

import java.io.IOException;

/**
 * An emergency implementation of {@link NameStore} to meet the needs for the LEGIH&CO network.
 *
 * @author C. Alexander Leigh
 */
public class EmergencyNameStore implements NameStore {
    @Override
    public Message doQuery(Message request) throws IOException {
        String name = request.getQuestion().getName().toString();

        Message reply = new Message(request.getHeader().getID());
        reply.addRecord(request.getQuestion(), Section.QUESTION);

        return switch (name) {
            case "fs1.leigh-co.com." -> buildReply(request, "10.52.4.34");
            case "ds1.leigh-co.com." -> buildReply(request, "10.52.4.11");
            case "bungie.leigh-co.com." -> buildReply(request, "10.52.4.35");
            case "db1.leigh-co.com." -> buildReply(request, "10.52.4.22");
            case "db2.leigh-co.com." -> buildReply(request, "10.52.4.21");
            case "dream.leigh-co.com." -> buildReply(request, "10.52.4.29");
            case "esp0.leigh-co.com." -> buildReply(request, "10.52.4.30");
            case "ghibli.leigh-co.com." -> buildReply(request, "10.52.4.44");
            case "gitlab.leigh-co.com." -> buildReply(request, "10.52.4.26");
            case "helium.leigh-co.com." -> buildReply(request, "10.52.4.24");
            case "irc0.leigh-co.com." -> buildReply(request, "10.52.4.67");
            case "m3.leigh-co.com." -> buildReply(request, "10.52.4.68");
            case "massive.leigh-co.com." -> buildReply(request, "10.52.4.28");
            case "mctest9.leigh-co.com." -> buildReply(request, "10.52.4.33");
            case "moon.leigh-co.com." -> buildReply(request, "10.52.4.32");
            case "mx0.leigh-co.com." -> buildReply(request, "10.52.4.27");
            case "pixel.leigh-co.com." -> buildReply(request, "10.52.4.19");
            case "slb0.leigh-co.com." -> buildReply(request, "10.52.4.51");
            case "ticket.leigh-co.com." -> buildReply(request, "10.52.4.10");
            case "web0.leigh-co.com." -> buildReply(request, "10.52.4.20");
            case "world.leigh-co.com." -> buildReply(request, "10.52.4.5");
            case "zebra.leigh-co.com." -> buildReply(request, "10.52.4.17");
            case "n1.leigh-co.com." -> buildReply(request, "10.52.4.59");
            case "n0.leigh-co.com." -> buildReply(request, "10.52.4.14");
            default -> null;
        };

    }

    private Message buildReply(Message request, String address) throws IOException {
        Message reply = new Message(request.getHeader().getID());
        reply.addRecord(request.getQuestion(), Section.QUESTION);
        reply.addRecord(Record.fromString(request.getQuestion().getName(), Type.A, DClass.IN,
                        86400, address, Name.root),
                Section.ANSWER);
        return reply;
    }
}
