/*
 *                                  Apache License
 *                            Version 2.0, January 2004
 *                         http://www.apache.org/licenses/
 *
 *    TERMS AND CONDITIONS FOR USE, REPRODUCTION, AND DISTRIBUTION
 *
 *    1. Definitions.
 *
 *       "License" shall mean the terms and conditions for use, reproduction,
 *       and distribution as defined by Sections 1 through 9 of this document.
 *
 *       "Licensor" shall mean the copyright owner or entity authorized by
 *       the copyright owner that is granting the License.
 *
 *       "Legal Entity" shall mean the union of the acting entity and all
 *       other entities that control, are controlled by, or are under common
 *       control with that entity. For the purposes of this definition,
 *       "control" means (i) the power, direct or indirect, to cause the
 *       direction or management of such entity, whether by contract or
 *       otherwise, or (ii) ownership of fifty percent (50%) or more of the
 *       outstanding shares, or (iii) beneficial ownership of such entity.
 *
 *       "You" (or "Your") shall mean an individual or Legal Entity
 *       exercising permissions granted by this License.
 *
 *       "Source" form shall mean the preferred form for making modifications,
 *       including but not limited to software source code, documentation
 *       source, and configuration files.
 *
 *       "Object" form shall mean any form resulting from mechanical
 *       transformation or translation of a Source form, including but
 *       not limited to compiled object code, generated documentation,
 *       and conversions to other media types.
 *
 *       "Work" shall mean the work of authorship, whether in Source or
 *       Object form, made available under the License, as indicated by a
 *       copyright notice that is included in or attached to the work
 *       (an example is provided in the Appendix below).
 *
 *       "Derivative Works" shall mean any work, whether in Source or Object
 *       form, that is based on (or derived from) the Work and for which the
 *       editorial revisions, annotations, elaborations, or other modifications
 *       represent, as a whole, an original work of authorship. For the purposes
 *       of this License, Derivative Works shall not include works that remain
 *       separable from, or merely link (or bind by name) to the interfaces of,
 *       the Work and Derivative Works thereof.
 *
 *       "Contribution" shall mean any work of authorship, including
 *       the original version of the Work and any modifications or additions
 *       to that Work or Derivative Works thereof, that is intentionally
 *       submitted to Licensor for inclusion in the Work by the copyright owner
 *       or by an individual or Legal Entity authorized to submit on behalf of
 *       the copyright owner. For the purposes of this definition, "submitted"
 *       means any form of electronic, verbal, or written communication sent
 *       to the Licensor or its representatives, including but not limited to
 *       communication on electronic mailing lists, source code control systems,
 *       and issue tracking systems that are managed by, or on behalf of, the
 *       Licensor for the purpose of discussing and improving the Work, but
 *       excluding communication that is conspicuously marked or otherwise
 *       designated in writing by the copyright owner as "Not a Contribution."
 *
 *       "Contributor" shall mean Licensor and any individual or Legal Entity
 *       on behalf of whom a Contribution has been received by Licensor and
 *       subsequently incorporated within the Work.
 *
 *    2. Grant of Copyright License. Subject to the terms and conditions of
 *       this License, each Contributor hereby grants to You a perpetual,
 *       worldwide, non-exclusive, no-charge, royalty-free, irrevocable
 *       copyright license to reproduce, prepare Derivative Works of,
 *       publicly display, publicly perform, sublicense, and distribute the
 *       Work and such Derivative Works in Source or Object form.
 *
 *    3. Grant of Patent License. Subject to the terms and conditions of
 *       this License, each Contributor hereby grants to You a perpetual,
 *       worldwide, non-exclusive, no-charge, royalty-free, irrevocable
 *       (except as stated in this section) patent license to make, have made,
 *       use, offer to sell, sell, import, and otherwise transfer the Work,
 *       where such license applies only to those patent claims licensable
 *       by such Contributor that are necessarily infringed by their
 *       Contribution(s) alone or by combination of their Contribution(s)
 *       with the Work to which such Contribution(s) was submitted. If You
 *       institute patent litigation against any entity (including a
 *       cross-claim or counterclaim in a lawsuit) alleging that the Work
 *       or a Contribution incorporated within the Work constitutes direct
 *       or contributory patent infringement, then any patent licenses
 *       granted to You under this License for that Work shall terminate
 *       as of the date such litigation is filed.
 *
 *    4. Redistribution. You may reproduce and distribute copies of the
 *       Work or Derivative Works thereof in any medium, with or without
 *       modifications, and in Source or Object form, provided that You
 *       meet the following conditions:
 *
 *       (a) You must give any other recipients of the Work or
 *           Derivative Works a copy of this License; and
 *
 *       (b) You must cause any modified files to carry prominent notices
 *           stating that You changed the files; and
 *
 *       (c) You must retain, in the Source form of any Derivative Works
 *           that You distribute, all copyright, patent, trademark, and
 *           attribution notices from the Source form of the Work,
 *           excluding those notices that do not pertain to any part of
 *           the Derivative Works; and
 *
 *       (d) If the Work includes a "NOTICE" text file as part of its
 *           distribution, then any Derivative Works that You distribute must
 *           include a readable copy of the attribution notices contained
 *           within such NOTICE file, excluding those notices that do not
 *           pertain to any part of the Derivative Works, in at least one
 *           of the following places: within a NOTICE text file distributed
 *           as part of the Derivative Works; within the Source form or
 *           documentation, if provided along with the Derivative Works; or,
 *           within a display generated by the Derivative Works, if and
 *           wherever such third-party notices normally appear. The contents
 *           of the NOTICE file are for informational purposes only and
 *           do not modify the License. You may add Your own attribution
 *           notices within Derivative Works that You distribute, alongside
 *           or as an addendum to the NOTICE text from the Work, provided
 *           that such additional attribution notices cannot be construed
 *           as modifying the License.
 *
 *       You may add Your own copyright statement to Your modifications and
 *       may provide additional or different license terms and conditions
 *       for use, reproduction, or distribution of Your modifications, or
 *       for any such Derivative Works as a whole, provided Your use,
 *       reproduction, and distribution of the Work otherwise complies with
 *       the conditions stated in this License.
 *
 *    5. Submission of Contributions. Unless You explicitly state otherwise,
 *       any Contribution intentionally submitted for inclusion in the Work
 *       by You to the Licensor shall be under the terms and conditions of
 *       this License, without any additional terms or conditions.
 *       Notwithstanding the above, nothing herein shall supersede or modify
 *       the terms of any separate license agreement you may have executed
 *       with Licensor regarding such Contributions.
 *
 *    6. Trademarks. This License does not grant permission to use the trade
 *       names, trademarks, service marks, or product names of the Licensor,
 *       except as required for reasonable and customary use in describing the
 *       origin of the Work and reproducing the content of the NOTICE file.
 *
 *    7. Disclaimer of Warranty. Unless required by applicable law or
 *       agreed to in writing, Licensor provides the Work (and each
 *       Contributor provides its Contributions) on an "AS IS" BASIS,
 *       WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
 *       implied, including, without limitation, any warranties or conditions
 *       of TITLE, NON-INFRINGEMENT, MERCHANTABILITY, or FITNESS FOR A
 *       PARTICULAR PURPOSE. You are solely responsible for determining the
 *       appropriateness of using or redistributing the Work and assume any
 *       risks associated with Your exercise of permissions under this License.
 *
 *    8. Limitation of Liability. In no event and under no legal theory,
 *       whether in tort (including negligence), contract, or otherwise,
 *       unless required by applicable law (such as deliberate and grossly
 *       negligent acts) or agreed to in writing, shall any Contributor be
 *       liable to You for damages, including any direct, indirect, special,
 *       incidental, or consequential damages of any character arising as a
 *       result of this License or out of the use or inability to use the
 *       Work (including but not limited to damages for loss of goodwill,
 *       work stoppage, computer failure or malfunction, or any and all
 *       other commercial damages or losses), even if such Contributor
 *       has been advised of the possibility of such damages.
 *
 *    9. Accepting Warranty or Additional Liability. While redistributing
 *       the Work or Derivative Works thereof, You may choose to offer,
 *       and charge a fee for, acceptance of support, warranty, indemnity,
 *       or other liability obligations and/or rights consistent with this
 *       License. However, in accepting such obligations, You may act only
 *       on Your own behalf and on Your sole responsibility, not on behalf
 *       of any other Contributor, and only if You agree to indemnify,
 *       defend, and hold each Contributor harmless for any liability
 *       incurred by, or claims asserted against, such Contributor by reason
 *       of your accepting any such warranty or additional liability.
 *
 *    END OF TERMS AND CONDITIONS
 *
 *    APPENDIX: How to apply the Apache License to your work.
 *
 *       To apply the Apache License to your work, attach the following
 *       boilerplate notice, with the fields enclosed by brackets "{}"
 *       replaced with your own identifying information. (Don't include
 *       the brackets!)  The text should be enclosed in the appropriate
 *       comment syntax for the file format. We also recommend that a
 *       file or class name and description of purpose be included on the
 *       same "printed page" as the copyright notice for easier
 *       identification within third-party archives.
 *
 *    Copyright {yyyy} {name of copyright owner}
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.mineacademy.fo.jsonsimple;

import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.Reader;

/* The following code was generated by JFlex 1.4.2 */

/**
 * @author FangYidong(fangyidong @ yahoo.com.cn)
 * @author Ralph Niemitz/RalleYTN(ralph.niemitz@gmx.de)
 * @version 2.0.0
 * @since 1.0.0
 */
public class Yylex {

	// ==== 11.03.2018 | Ralph Niemitz/RalleYTN(ralph.niemitz@gmx.de)
	// -	Properly formatted this class
	// -	Replaced switch-case block with if-else blocks because they are faster
	// ====

	public static final int YYINITIAL = 0;
	public static final int STRING_BEGIN = 2;
	public static final int YYEOF = -1;

	private static final int ZZ_BUFFERSIZE = 16384;
	private static final int ZZ_LEXSTATE[] = { 0, 0, 1, 1 };
	private static final String ZZ_CMAP_PACKED = "\11\0\1\7\1\7\2\0\1\7\22\0\1\7\1\0\1\11\10\0\1\6\1\31\1\2\1\4\1\12\12\3\1\32\6\0\4\1\1\5\1\1\24\0\1\27\1\10\1\30\3\0\1\22\1\13\2\1\1\21\1\14\5\0\1\23\1\0\1\15\3\0\1\16\1\24\1\17\1\20\5\0\1\25\1\0\1\26\uff82\0";
	private static final char[] ZZ_CMAP = Yylex.zzUnpackCMap(Yylex.ZZ_CMAP_PACKED);
	private static final int[] ZZ_ACTION = Yylex.zzUnpackAction();
	private static final String ZZ_ACTION_PACKED_0 = "\2\0\2\1\1\2\1\3\1\4\3\1\1\5\1\6\1\7\1\10\1\11\1\12\1\13\1\14\1\15\5\0\1\14\1\16\1\17\1\20\1\21\1\22\1\23\1\24\1\0\1\25\1\0\1\25\4\0\1\26\1\27\2\0\1\30";
	private static final int[] ZZ_ROWMAP = Yylex.zzUnpackRowMap();
	private static final String ZZ_ROWMAP_PACKED_0 = "\0\0\0\33\0\66\0\121\0\154\0\207\0\66\0\242\0\275\0\330\0\66\0\66\0\66\0\66\0\66\0\66\0\363\0\u010e\0\66\0\u0129\0\u0144\0\u015f\0\u017a\0\u0195\0\66\0\66\0\66\0\66\0\66\0\66\0\66\0\66\0\u01b0\0\u01cb\0\u01e6\0\u01e6\0\u0201\0\u021c\0\u0237\0\u0252\0\66\0\66\0\u026d\0\u0288\0\66";
	private static final int ZZ_UNKNOWN_ERROR = 0;
	private static final int ZZ_NO_MATCH = 1;
	private static final int ZZ_PUSHBACK_2BIG = 2;
	private static final String[] ZZ_ERROR_MSG = { "Unkown internal scanner error", "Error: could not match input", "Error: pushback value was too large" };
	private static final int[] ZZ_ATTRIBUTE = Yylex.zzUnpackAttribute();
	private static final String ZZ_ATTRIBUTE_PACKED_0 = "\2\0\1\11\3\1\1\11\3\1\6\11\2\1\1\11\5\0\10\11\1\0\1\1\1\0\1\1\4\0\2\11\2\0\1\11";
	private static final int[] ZZ_TRANS = {

			2, 2, 3, 4, 2, 2, 2, 5, 2, 6,
			2, 2, 7, 8, 2, 9, 2, 2, 2, 2,
			2, 10, 11, 12, 13, 14, 15, 16, 16, 16,
			16, 16, 16, 16, 16, 17, 18, 16, 16, 16,
			16, 16, 16, 16, 16, 16, 16, 16, 16, 16,
			16, 16, 16, 16, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, 4, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
			-1, 4, 19, 20, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, 20, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 5, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
			21, -1, -1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, 22, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
			23, -1, -1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, 16, 16, 16, 16, 16, 16, 16,
			16, -1, -1, 16, 16, 16, 16, 16, 16, 16,
			16, 16, 16, 16, 16, 16, 16, 16, 16, 16,
			-1, -1, -1, -1, -1, -1, -1, -1, 24, 25,
			26, 27, 28, 29, 30, 31, 32, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
			33, -1, -1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, 34, 35, -1, -1,
			34, -1, -1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
			36, -1, -1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, 37, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
			-1, 38, -1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, 39, -1, 39, -1, 39, -1, -1,
			-1, -1, -1, 39, 39, -1, -1, -1, -1, 39,
			39, -1, -1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 33, -1, 20, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, 20, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1, -1, 35,
			-1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, 38, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1, -1, 40,
			-1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, 41, -1, -1, -1, -1, -1,
			-1, -1, -1, -1, -1, 42, -1, 42, -1, 42,
			-1, -1, -1, -1, -1, 42, 42, -1, -1, -1,
			-1, 42, 42, -1, -1, -1, -1, -1, -1, -1,
			-1, -1, 43, -1, 43, -1, 43, -1, -1, -1,
			-1, -1, 43, 43, -1, -1, -1, -1, 43, 43,
			-1, -1, -1, -1, -1, -1, -1, -1, -1, 44,
			-1, 44, -1, 44, -1, -1, -1, -1, -1, 44,
			44, -1, -1, -1, -1, 44, 44, -1, -1, -1,
			-1, -1, -1, -1, -1,
	};

	private Reader zzReader;
	private int zzLexicalState = Yylex.YYINITIAL;
	private char zzBuffer[] = new char[Yylex.ZZ_BUFFERSIZE];
	private int zzMarkedPos;
	private int zzCurrentPos;
	private int zzStartRead;
	private int zzEndRead;
	private int yychar;
	private boolean zzAtEOF;
	private StringBuffer sb = new StringBuffer();

	private static int[] zzUnpackAction() {

		final int[] result = new int[45];
		int offset = 0;
		offset = Yylex.zzUnpackAction(Yylex.ZZ_ACTION_PACKED_0, offset, result);
		return result;
	}

	private static int zzUnpackAction(String packed, int offset, int[] result) {

		int i = 0;
		int j = offset;
		final int l = packed.length();

		while (i < l) {

			int count = packed.charAt(i++);
			final int value = packed.charAt(i++);

			do
				result[j++] = value;
			while (--count > 0);
		}

		return j;
	}

	private static int[] zzUnpackRowMap() {

		final int[] result = new int[45];
		int offset = 0;
		offset = Yylex.zzUnpackRowMap(Yylex.ZZ_ROWMAP_PACKED_0, offset, result);
		return result;
	}

	private static int zzUnpackRowMap(String packed, int offset, int[] result) {

		int i = 0;
		int j = offset;
		final int l = packed.length();

		while (i < l) {

			final int high = packed.charAt(i++) << 16;
			result[j++] = high | packed.charAt(i++);
		}

		return j;
	}

	private static int[] zzUnpackAttribute() {

		final int[] result = new int[45];
		int offset = 0;
		offset = Yylex.zzUnpackAttribute(Yylex.ZZ_ATTRIBUTE_PACKED_0, offset, result);
		return result;
	}

	private static int zzUnpackAttribute(String packed, int offset, int[] result) {

		int i = 0;
		int j = offset;
		final int l = packed.length();

		while (i < l) {

			int count = packed.charAt(i++);
			final int value = packed.charAt(i++);

			do
				result[j++] = value;
			while (--count > 0);
		}

		return j;
	}

	public int getPosition() {

		return this.yychar;
	}

	public Yylex(Reader in) {

		this.zzReader = in;
	}

	public Yylex(InputStream in) {

		this(new InputStreamReader(in));
	}

	private static char[] zzUnpackCMap(String packed) {

		final char[] map = new char[0x10000];
		int i = 0;
		int j = 0;

		while (i < 90) {

			int count = packed.charAt(i++);
			final char value = packed.charAt(i++);

			do
				map[j++] = value;
			while (--count > 0);
		}

		return map;
	}

	private boolean zzRefill() throws IOException {

		if (this.zzStartRead > 0) {

			System.arraycopy(this.zzBuffer, this.zzStartRead, this.zzBuffer, 0, this.zzEndRead - this.zzStartRead);

			this.zzEndRead -= this.zzStartRead;
			this.zzCurrentPos -= this.zzStartRead;
			this.zzMarkedPos -= this.zzStartRead;
			this.zzStartRead = 0;
		}

		if (this.zzCurrentPos >= this.zzBuffer.length) {

			final char[] newBuffer = new char[this.zzCurrentPos * 2];
			System.arraycopy(this.zzBuffer, 0, newBuffer, 0, this.zzBuffer.length);
			this.zzBuffer = newBuffer;
		}

		final int numRead = this.zzReader.read(this.zzBuffer, this.zzEndRead, this.zzBuffer.length - this.zzEndRead);

		if (numRead > 0) {

			this.zzEndRead += numRead;
			return false;
		}

		if (numRead == 0) {

			final int c = this.zzReader.read();

			if (c == -1)
				return true;
			else {

				this.zzBuffer[this.zzEndRead++] = (char) c;
				return false;
			}
		}

		return true;
	}

	public final void yyclose() throws IOException {

		this.zzAtEOF = true; // indicate end of file
		this.zzEndRead = this.zzStartRead; // invalidate buffer

		if (this.zzReader != null)
			this.zzReader.close();
	}

	public final void yyreset(Reader reader) {

		this.zzReader = reader;
		this.zzAtEOF = false;
		this.zzEndRead = this.zzStartRead = 0;
		this.zzCurrentPos = this.zzMarkedPos = 0;
		this.yychar = 0;
		this.zzLexicalState = Yylex.YYINITIAL;
	}

	public final int yystate() {

		return this.zzLexicalState;
	}

	public final void yybegin(int newState) {

		this.zzLexicalState = newState;
	}

	public final String yytext() {

		return new String(this.zzBuffer, this.zzStartRead, this.zzMarkedPos - this.zzStartRead);
	}

	public final char yycharat(int pos) {

		return this.zzBuffer[this.zzStartRead + pos];
	}

	public final int yylength() {

		return this.zzMarkedPos - this.zzStartRead;
	}

	private void zzScanError(int errorCode) {

		String message;

		try {

			message = Yylex.ZZ_ERROR_MSG[errorCode];

		} catch (final ArrayIndexOutOfBoundsException exception) {

			message = Yylex.ZZ_ERROR_MSG[Yylex.ZZ_UNKNOWN_ERROR];
		}

		// ==== 17.03.2018 | Ralph Niemitz/RalleYTN(ralph.niemitz@gmx.de)
		// DO NOT CHANGE!
		// Even if Codacy says so.
		// ====

		throw new Error(message);
	}

	public void yypushback(int number) {

		if (number > yylength())
			this.zzScanError(Yylex.ZZ_PUSHBACK_2BIG);

		this.zzMarkedPos -= number;
	}

	public Yytoken yylex() throws IOException, JSONParseException {

		int zzInput;
		int zzAction;
		int zzCurrentPosL;
		int zzMarkedPosL;
		int zzEndReadL = this.zzEndRead;
		char[] zzBufferL = this.zzBuffer;
		final char[] zzCMapL = Yylex.ZZ_CMAP;
		final int[] zzTransL = Yylex.ZZ_TRANS;
		final int[] zzRowMapL = Yylex.ZZ_ROWMAP;
		final int[] zzAttrL = Yylex.ZZ_ATTRIBUTE;

		while (true) {

			zzMarkedPosL = this.zzMarkedPos;
			this.yychar += zzMarkedPosL - this.zzStartRead;
			zzAction = -1;
			zzCurrentPosL = this.zzCurrentPos = this.zzStartRead = zzMarkedPosL;
			int zzState = Yylex.ZZ_LEXSTATE[this.zzLexicalState];

			zzForAction:
			{

				while (true) {

					if (zzCurrentPosL < zzEndReadL)
						zzInput = zzBufferL[zzCurrentPosL++];
					else if (this.zzAtEOF) {

						zzInput = Yylex.YYEOF;
						break zzForAction;

					} else {

						// store back cached positions
						this.zzCurrentPos = zzCurrentPosL;
						this.zzMarkedPos = zzMarkedPosL;
						final boolean eof = this.zzRefill();

						// get translated positions and possibly new buffer
						zzCurrentPosL = this.zzCurrentPos;
						zzMarkedPosL = this.zzMarkedPos;
						zzBufferL = this.zzBuffer;
						zzEndReadL = this.zzEndRead;

						if (eof) {

							zzInput = Yylex.YYEOF;
							break zzForAction;

						} else
							zzInput = zzBufferL[zzCurrentPosL++];
					}

					final int zzNext = zzTransL[zzRowMapL[zzState] + zzCMapL[zzInput]];

					if (zzNext == -1)
						break zzForAction;

					zzState = zzNext;

					final int zzAttributes = zzAttrL[zzState];

					if ((zzAttributes & 1) == 1) {

						zzAction = zzState;
						zzMarkedPosL = zzCurrentPosL;

						if ((zzAttributes & 8) == 8)
							break zzForAction;
					}
				}
			}

			// store back cached position
			this.zzMarkedPos = zzMarkedPosL;
			final int value = zzAction < 0 ? zzAction : ZZ_ACTION[zzAction];

			if (value < 25 || value > 48)
				if (value == 11)
					this.sb.append(this.yytext());
				else if (value == 4) {

					this.sb = new StringBuffer();
					this.yybegin(Yylex.STRING_BEGIN);

				} else if (value == 16)
					this.sb.append('\b');
				else if (value == 6)
					return new Yytoken(Yytoken.TYPE_RIGHT_BRACE, null);
				else if (value == 23)
					return new Yytoken(Yytoken.TYPE_VALUE, Boolean.valueOf(this.yytext()));
				else if (value == 22)
					return new Yytoken(Yytoken.TYPE_VALUE, null);
				else if (value == 13) {

					this.yybegin(Yylex.YYINITIAL);
					return new Yytoken(Yytoken.TYPE_VALUE, this.sb.toString());

				} else if (value == 12)
					this.sb.append('\\');
				else if (value == 21)
					return new Yytoken(Yytoken.TYPE_VALUE, Double.valueOf(yytext()));
				else if (value == 1)
					throw new JSONParseException(this.yychar, JSONParseException.ERROR_UNEXPECTED_CHAR, this.yycharat(0));
				else if (value == 8)
					return new Yytoken(Yytoken.TYPE_RIGHT_SQUARE, null);
				else if (value == 19)
					this.sb.append('\r');
				else if (value == 15)
					this.sb.append('/');
				else if (value == 10)
					return new Yytoken(Yytoken.TYPE_COLON, null);
				else if (value == 14)
					this.sb.append('"');
				else if (value == 5)
					return new Yytoken(Yytoken.TYPE_LEFT_BRACE, null);
				else if (value == 17)
					this.sb.append('\f');
				else if (value == 24)
					try {

						final int ch = Integer.parseInt(this.yytext().substring(2), 16);
						this.sb.append((char) ch);

					} catch (final Exception exception) {

						throw new JSONParseException(this.yychar, JSONParseException.ERROR_UNEXPECTED_EXCEPTION, exception);
					}
				else if (value == 20)
					this.sb.append('\t');
				else if (value == 7)
					return new Yytoken(Yytoken.TYPE_LEFT_SQUARE, null);
				else if (value == 2)
					return new Yytoken(Yytoken.TYPE_VALUE, Long.valueOf(this.yytext()));
				else if (value == 18)
					this.sb.append('\n');
				else if (value == 9)
					return new Yytoken(Yytoken.TYPE_COMMA, null);
				else if (value == 3) {

				} else if (zzInput == Yylex.YYEOF && this.zzStartRead == this.zzCurrentPos) {

					this.zzAtEOF = true;
					return null;

				} else
					this.zzScanError(Yylex.ZZ_NO_MATCH);
		}
	}
}
